"""
Main entry point for the DSPy prompt optimization module.

This allows running the module as:
python -m gptme.eval.dspy [command] [options]
"""

from .cli import main

if __name__ == "__main__":
    main()
